function [W,dt] = adjacency_weighted(DATA, nn, t)

% Compute the weighted adjacency matrix of the data set DATA with
% (possibly self-tuning) gaussian kernel parameter. See 
% http://webee.technion.ac.il/~lihi/Demos/SelfTuningClustering.html
%
% W = adjacency_weighted(DATA, nn, t);
% 
% DATA - NxK matrix. Data points are rows. 
% nn - integer: number of nearest neighbors
% t - (typically) integer: parameter for gaussian kernel
%                          if not provided, self-tuning is applied
%
% Returns: W, sparse symmetric NxN matrix of distances between the
% adjacent points. 
%
% Example: 
% 
% W = adjacency(X,6) 
%   W contains the adjacency matrix for the data set X. For each 
%   point, the weights wrt to its 6 adjacent points are stored. 
%   Note that all the points which are not in the top nn are not
%   considered as connected at all.
%
% Note: the adjacency relation is symmetrized, i.e. if
% point a is adjacent to point b, then point b is also considered to be
% adjacent to point a.
%
% Code has been tested to run on Octave (v.3.2.4)
%
% The original code this script has been built from is by Mikhail
% Belkin (misha@math.uchicago.edu) and can be found at
% http://www.mathworks.com/matlabcentral/fileexchange/29227-eigen-function-of-the-laplacian/content/Diffusion_Family/Diffusion%20Family/adjacency.m
%
% This version has been developed by:
% Davide Eynard
% davide.eynard@usi.ch



if (nargin < 2) || ~isreal(nn)
  fprintf('ERROR: Too few arguments given or incorrect arguments.\n');
  return;
end

if (nargin < 3)
  t = 0; % assume self-tuning gaussian kernel if no t is specified
end

n = size(DATA,1);
fprintf ('DATA: %d points in %d dimensional space.\n',n,size (DATA,2));
fprintf('Creating the adjacency matrix. Nearest neighbors, N=%d.\n', nn); 
  
W = sparse(n,n);

% NOTE: L2_distance is WAY FASTER than pdist!!!
dt = L2_distance(DATA',DATA');
[Z,I] = sort (dt,2);

for i=1:n
    if ( mod(i,500) == 0) 
        fprintf('%d points processed.\n', i);
    end
    for j=2:nn+1
        if (t)
            % use gaussian kernel with t as a parameter
            W(i,I(i,j))= exp(-Z(i,j).^2/t);
        else
            % use self-tuning
            W(i,I(i,j))= exp(-Z(i,j).^2/(Z(i,nn+1)*Z(j,nn+1)));
        end
        W(I(i,j),i)= W(i,I(i,j));
    end    
end
    
end
