function spectralDemo(dataset,nn,t,interactive)

% if you want to have the eigenspace to play with when you exit from this
% function, use the following line as function header:
%
%function [X,classes,clusters,V] = spectralDemo(dataset,nn,t,interactive)

% ------------------------ parameters section -------------------------------
if nargin<4
	% to make the demo non-interactive (skip keypresses) just set this to 0
	interactive = 1;
end
if nargin<3
	t = 0; % parameter for gaussian Kernel (0=self-tuning)
end

if nargin<2
	nn = 15; % number of nearest neighbors used to build W
end

if nargin<1
	dataset = './data/circles01.mat';
end

% ------------------------ here be the code ---------------------------------
% load dataset - also try with other .mat files (e.g. circles02.mat, ...)
load (dataset);

fprintf('I have loaded dataset %s\n',dataset);
fprintf('Dataset characteristics: %d points, %d-dimensional space, %d clusters\n\n',size(X,1),size(X,2),clusters);

% plot data
fprintf('Plotting data...\n');
figure;plotClustering(X,classes,classes);drawnow();
title('Data plot (colors/shapes represent different classes)')
if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

% calculate weight matrix (also get --optional-- distances matrix for comparison)
fprintf('Calculating weight matrix...\n');
[W,dt] = adjacency_weighted(X,nn,t);

fprintf('Showing distances between points.\n');
figure;imagesc(dt),colorbar;
title('Distances between points (blue=near, red=far)');
fprintf('Note that, depending on the dataset, relying on distances in\n');
fprintf('the data space might not be the best choice. Look for instance\n');
fprintf('at what happens with circles and crescents examples: points\n');
fprintf('which are clearly in different clusters look close, while others\n');
fprintf('belonging to the same clusters are actually far apart.\n');
fprintf('In these cases, is the concept of "similarity" equivalent to the\n');
fprintf('concept of "distance"?\n\n');
if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

fprintf('Showing Weight matrix W.\n');
figure;imagesc(full(W)),colorbar;
title(sprintf('Weight matrix W, nn=%d, t=%d',nn,t));
fprintf('Note that, instead, working on the weights matrix allows you to\n');
fprintf('work on a different concept of similarity, which is related not\n');
fprintf('just to distance but also to the connectivity between points.\n');
fprintf('In the algorithm we use (see adjacency_weighted.m), two points\n');
fprintf('are connected if at least one of them appears in the other''s\n');
fprintf('neighborhood. In this case, their similarity is calculated as\n');
fprintf('a Gaussian Kernel.\n'); 
if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

fprintf('Building normalized laplacian...\n');
[L,invSqD] = laplacian(W);

% get eigenvalues 
[V,E] = eigs(L,10,'SR'); 
% ... why 5? Well, so we can see the difference (if any) between the first k
% (k=#clusters) eigenvalues and the others (see e below). Moreover, plotting 
% the eigenspace in higher dimensions is funny :-)

% normalize and sort V according to the (ascending) order of eigenvalues
V = invSqD * V;
[eigenvals,i] = sort(diag(E));
V = V(:,i);
figure;plot(eigenvals);
fprintf('This is the plot of Laplacian''s eigenvalues, ordered by magnitude\n');
fprintf('(smallest first). The more points are naturally clustered, the more\n');
fprintf('the first eigenvalues will be close to 0 and far from the others.\n');
fprintf('The number of eigenvalues equal (in the ideal case) or close (in the\n');
fprintf('real case) to zero equals the number of connected components in the\n');
fprintf('adjacency graph, i.e. the number of clusters in the dataset.\n'); 

if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

% run kmeans twice: the first time on the original dataset...
fprintf('Running k-means on the original dataset...\n');
[labelsKM,centroid,sseKM]= repKmeans(X, clusters,10);
figure;plotClustering(X,labelsKM,classes);
title(sprintf('Results of k-means (k=%d) ran on the original data space. Accuracy: %.4f, SSE: %.4f',clusters,accuracy(labelsKM,classes),sseKM));

if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

% ... and the second time on the eigenspace built with the first K 
% eigenvectors (the variable named "clusters" is holding this value)
fprintf('Running k-means on the eigenspace...\n');
[labelsSC,centroid,sseSC]= repKmeans(V(:,1:clusters), clusters,10);
figure;plotClustering(X,labelsSC,classes);
title(sprintf('Results of k-means (k=%d) ran on the k-dimensional eigenspace. Accuracy: %.4f, SSE: %.4f',clusters,accuracy(labelsSC,classes),sseSC));
fprintf('Note that, even if you get the same level of accuracy with\n');
fprintf('k-means and spectral (i.e. you cluster points in the same way),\n');
fprintf('SSE might be very different. This is due to the fact that\n');
fprintf('distances in the spectral case are calculated in the eigenspace\n');
fprintf('and not in the origina data space.\n');
if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

fprintf('Was the spectral clustering result not satisfying yet?\n');
fprintf('Try to run the demo again by tweaking its parameters (nn,t).\n');
fprintf('(NOTE: do not increase nn too much, otherwise computations\n');
fprintf('will take a looong time!)\n\n');
end
